package ch.rakudave.jnetmap.controller;

import java.util.Collection;
import java.util.HashMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import ch.rakudave.jnetmap.model.Map;
import ch.rakudave.jnetmap.model.device.Device;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.logging.Logger;
import ch.rakudave.jnetmap.view.components.StatusBar;

public class StatusUpdater {
	private static java.util.Map<Map, ScheduledFuture<?>> maps = new HashMap<Map, ScheduledFuture<?>>();

	public static void addMap(final Map m) {
		Logger.debug("Adding scheduled task for "+m.getFileName()+", running every "+m.getUpdateInterval()+"m");
		if (!maps.containsKey(m)) {
			Scheduler.scheduleAtFixedRate(new Runnable() {
				@Override
				public void run() {
					refresh(m);
				}
			}, 0, (int)(m.getUpdateInterval()*60), TimeUnit.SECONDS);
		}
	}

	public static void removeMap(Map m) {
		if (maps.containsKey(m)) {
			maps.get(m).cancel(true);
		}
	}

	public static void updateTimeInterval(Map m) {
		if (maps.containsKey(m)) {
			removeMap(m);
			addMap(m);
		}
	}

	public static void refresh(Map m) {
		Logger.debug("Updating devices on map "+m.getFileName());
		StatusBar.getInstance().setBusy(true);
		ExecutorService es = Executors.newFixedThreadPool(Settings.getInt("status.update.threads", 5));
		final Collection<Device> vertecies = m.getVertices();
		final CountDownLatch latch = new CountDownLatch(vertecies.size());
		StatusBar.getInstance().setMessage(Lang.getNoHTML("message.status.update").replaceAll("%name%", m.getFileName()+".jnm"));
		for (final Device d : vertecies) {
			es.submit(new Runnable() {
				@Override
				public void run() {
					try {
						d.updateStatus();
						StatusBar.getInstance().setProgress((int) (((vertecies.size() - latch.getCount()) / (float) vertecies.size()) * 100.0));
					} catch (Exception e) {
						Logger.error("Unable to update device " + d.getName(), e);
					} finally {
						latch.countDown();
					}
				}
			});
		}
		es.shutdown();
		try {
			latch.await();
		} catch (InterruptedException e) {
			Logger.debug("Failed to await map refresh latch", e);
		} finally {
			StatusBar.getInstance().clearMessage();
			StatusBar.getInstance().setProgress(-1);
			StatusBar.getInstance().setBusy(false);
		}
	}
}
