package ch.rakudave.jnetmap.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Properties;

import ch.rakudave.jnetmap.util.logging.Logger;

/**
 * @author rakudave
 */
public class Lang {
	private static Properties lang, fallback;
	private static String currentLanguage = Settings.get("lang", "English");

	public static String currentLanguage() {
		return currentLanguage;
	}
	
	/**
	 * @param key id
	 * @return translated string (in the current language) of the key if not found
	 */
	public static String get(String key) {
		if (lang == null) load();
		String value = lang.getProperty(key);
		if (value == null) {
			Logger.warn("No translation for " + key + " in " + currentLanguage);
			value = fallback.getProperty(key);
			if (value == null) return key;
		}
		return value;
	}
	
	public static String[] getLanguages() {
		String[] files = IO.langDir.list(new FilenameFilter() {
			@Override
			public boolean accept(File arg0, String arg1) {
				return !(arg1.startsWith(".") || arg1.endsWith("~"));
			}
		});
		if (files == null) return new String[] {};
		Arrays.sort(files);
		return files;
	}
	
	public static String getNoHTML(String key) {
		return get(key).replaceAll("\\<.*?>", "")
				.replaceAll("&auml;", "ä")
				.replaceAll("&Auml;", "Ä")
				.replaceAll("&ouml;", "ö")
				.replaceAll("&Ouml;", "Ö")
				.replaceAll("&uuml;", "ü")
				.replaceAll("&Uuml;", "Ü"); //meh
		//TODO find better way to convert from html
	}
	
	/**
	 * Loads the preferred language or English if none was defined
	 */
	public static void load() {
		load(currentLanguage);
	}
	
	/**
	 * Load a specified language, looks for a file with the same name in the /lang directory
	 * @param language
	 */
	public static synchronized void load(String language) {
		Logger.info("Loading language: " + language);
		if (fallback == null) {
			fallback = new Properties();
			load("English");
			fallback = lang;
		}
		lang = new Properties();
		try {
			lang.load(new FileInputStream(IO.langDir.getAbsolutePath() + "/" + language));
			for (Object key : lang.keySet()) {
				// Properties.load() thinks it's cool to use Latin-1. I don't.
				String value = new String(((String) lang.get(key)).getBytes("iso8859-1"), "utf-8");
				lang.put(key, "<html>"+value+"</html>");
			}
			currentLanguage = language;
			Settings.put("lang", currentLanguage);
		} catch (IOException e) {
			Logger.error("Failed to open language-file " + language, e);
		}
	}
	
	private Lang() {}
}
